@@ -17,6 +17,7 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
+#include "boxcompat.h"
 #include "dlg.h"
 #include "display.h"
 #include "globalopts.h"
@@ -570,43 +571,6 @@ int multi_tab_get_page_num(MultiTab *tab)
             tab->widget);
 }
 
-MultiTab *multi_tab_get_tab_under_pointer(int x, int y)
-{
-    MultiWin *win = multi_win_get_win_under_pointer();
-    GList *tab_node;
-    GtkPositionType tab_pos;
-    gboolean vert_tabs;
-    GtkAllocation allocation;
-
-    if (!win)
-        return NULL;
-    if (win->ntabs == 1)
-        return win->tabs->data;
-    tab_pos = win->tab_pos;
-    vert_tabs = (tab_pos == GTK_POS_LEFT || tab_pos == GTK_POS_RIGHT);
-    for (tab_node = win->tabs; tab_node; tab_node = g_list_next(tab_node))
-    {
-        int x0, y0, x1, y1;
-        MultiTab *tab = tab_node->data;
-        GtkWidget *label = gtk_notebook_get_tab_label(
-                GTK_NOTEBOOK(win->notebook), tab->widget);
-
-        if (!gtk_widget_get_mapped(label))
-            continue;
-        gdk_window_get_origin(gtk_widget_get_window(label), &x0, &y0);
-        gtk_widget_get_allocation(label, &allocation);
-        x0 += allocation.x;
-        y0 += allocation.y;
-        x1 = x0 + allocation.width;
-        y1 = y0 + allocation.height;
-        if (vert_tabs && y >= y0 && y < y1)
-            return tab;
-        else if (!vert_tabs && x >= x0 && x < x1)
-            return tab;
-    }
-    return NULL;
-}
-
 MultiWin *multi_tab_get_parent(MultiTab *tab)
 {
     return tab->parent;
@@ -837,7 +801,9 @@ static void multi_win_pack_for_multiple_tabs(MultiWin *win)
 #endif
     for (link = win->tabs; link; link = g_list_next(link))
     {
-        multi_tab_pack_for_multiple(link->data, nb);
+        MultiTab *tab = link->data;
+        
+        multi_tab_pack_for_multiple(tab, nb);
     }
 }
 
@@ -920,8 +886,12 @@ static void add_menu_bar(MultiWin *win)
     if (win->show_menu_bar)
         return;
     menu_bar = menutree_get_top_level_widget(win->menu_bar);
+#if GTK_CHECK_VERSION(3, 0, 0)
+    gtk_grid_attach(GTK_GRID(win->vbox), menu_bar, 0, -1, 1, 1);
+#else
     gtk_box_pack_start(GTK_BOX(win->vbox), menu_bar, FALSE, FALSE, 0);
     gtk_box_reorder_child(GTK_BOX(win->vbox), menu_bar, 0);
+#endif
     gtk_widget_show(menu_bar);
     win->show_menu_bar = TRUE;
 }
@@ -1285,8 +1255,7 @@ static void multi_win_name_tab_action(MultiWin * win)
 
     tab->rename_dialog = dialog_w;
     gtk_dialog_set_default_response(dialog, GTK_RESPONSE_APPLY);
-    gtk_box_pack_start(GTK_BOX(gtk_dialog_get_content_area(dialog)),
-            name_w, TRUE, TRUE, 8);
+    box_compat_packv(gtk_dialog_get_content_area(dialog), name_w, TRUE, 8);
     gtk_entry_set_activates_default(name_e, TRUE);
     gtk_entry_set_text(name_e, name ? name : "");
     gtk_widget_show_all(dialog_w);
@@ -1327,7 +1296,6 @@ static void multi_win_set_window_title_action(MultiWin * win)
     GtkDialog *dialog = GTK_DIALOG(dialog_w);
     GtkWidget *title_w = gtk_entry_new();
     GtkEntry *title_e = GTK_ENTRY(title_w);
-    GtkWidget *hbox = gtk_hbox_new(FALSE, 8);
     GtkWidget *img = gtk_image_new_from_stock(GTK_STOCK_DIALOG_INFO,
             GTK_ICON_SIZE_DIALOG);
     GtkWidget *tip_label = gtk_label_new(_("The title string may include '%s' "
@@ -1338,14 +1306,19 @@ static void multi_win_set_window_title_action(MultiWin * win)
             "profile's title string."));
     int response;
     const char *title;
-    GtkBox *content_area = GTK_BOX(gtk_dialog_get_content_area(dialog));
+    GtkWidget *content_area = gtk_dialog_get_content_area(dialog);
+#if GTK_CHECK_VERSION(3, 0, 0)
+    GtkWidget *hbox = gtk_grid_new();
+#else
+    GtkWidget *hbox = gtk_hbox_new(FALSE, 8);
+#endif
 
     gtk_dialog_set_default_response(dialog, GTK_RESPONSE_APPLY);
-    gtk_box_pack_start(content_area, title_w, FALSE, TRUE, 8);
-    gtk_box_pack_start(GTK_BOX(hbox), img, FALSE, TRUE, 0);
+    box_compat_packv(content_area, title_w, FALSE, 8);
+    box_compat_packh(hbox, img, FALSE, 8);
     gtk_label_set_line_wrap(GTK_LABEL(tip_label), TRUE);
-    gtk_box_pack_start(GTK_BOX(hbox), tip_label, TRUE, TRUE, 0);
-    gtk_box_pack_start(content_area, hbox, TRUE, TRUE, 8);
+    box_compat_packh(hbox, tip_label, TRUE, 8);
+    box_compat_packv(content_area, hbox, TRUE, 8);
     gtk_entry_set_activates_default(title_e, TRUE);
     gtk_entry_set_text(title_e, win->title_template ? win->title_template : "");
     gtk_widget_show_all(dialog_w);
@@ -1572,6 +1545,15 @@ static void multi_win_move_tab_right_action(MultiWin *win)
     multi_win_move_tab_by_one(win, 1);
 }
 
+static void multi_win_close_window_action(MultiWin *win)
+{
+    if (!multi_win_delete_handler ||
+            !multi_win_delete_handler(win->gtkwin, (GdkEvent *) win, win))
+    {
+        multi_win_destructor(win, TRUE);
+    }
+}
+
 static void multi_win_popup_new_term_with_profile(GtkMenu *menu)
 {
     gtk_menu_popup(menu, NULL, NULL, NULL, NULL, 0, GDK_CURRENT_TIME);
@@ -1582,7 +1564,7 @@ static void multi_win_connect_actions(MultiWin * win)
     multi_win_menu_connect_swapped(win, MENUTREE_FILE_NEW_WINDOW, G_CALLBACK
         (multi_win_new_window_action), win, NULL, NULL, NULL);
     multi_win_menu_connect_swapped(win, MENUTREE_FILE_CLOSE_WINDOW, G_CALLBACK
-        (multi_win_delete), win, NULL, NULL, NULL);
+        (multi_win_close_window_action), win, NULL, NULL, NULL);
     multi_win_menu_connect_swapped(win, MENUTREE_FILE_NEW_TAB, G_CALLBACK
         (multi_win_new_tab_action), win, NULL, NULL, NULL);
     multi_win_menu_connect_swapped(win, MENUTREE_FILE_CLOSE_TAB, G_CALLBACK
@@ -1741,6 +1723,7 @@ static void multi_win_composited_changed(GtkWidget *widget, MultiWin *win)
             
             gtk_window_move(GTK_WINDOW(win->gtkwin), x, y);
             gtk_widget_realize(win->gtkwin);
+            dwin = gtk_widget_get_window(win->gtkwin);
             gdk_x11_window_set_user_time(dwin, user_time);
             if (was_minimized)
                 gtk_window_iconify(GTK_WINDOW(win->gtkwin));
@@ -1847,7 +1830,13 @@ MultiWin *multi_win_new_blank(const char *display_name, Options *shortcuts,
             G_CALLBACK(multi_win_map_event_handler), win);
 #endif
 
+#if GTK_CHECK_VERSION(3, 0, 0)
+    win->vbox = gtk_grid_new();
+    gtk_orientable_set_orientation(GTK_ORIENTABLE(win->vbox),
+            GTK_ORIENTATION_VERTICAL);
+#else
     win->vbox = gtk_vbox_new(FALSE, 0);
+#endif
     gtk_container_add(GTK_CONTAINER(win->gtkwin), win->vbox);
 
     options_ref(shortcuts);
@@ -1922,7 +1911,7 @@ MultiWin *multi_win_new_blank(const char *display_name, Options *shortcuts,
         multi_win_set_show_tabs_menu_items(win, FALSE);
         multi_win_hide_tabs(win);
     }
-    gtk_box_pack_start(GTK_BOX(win->vbox), win->notebook, TRUE, TRUE, 0);
+    box_compat_packv(win->vbox, win->notebook, TRUE, 0);
     g_signal_connect(win->notebook, "switch-page",
         G_CALLBACK(multi_win_page_switched), win);
     g_signal_connect(win->gtkwin, "focus-in-event",
@@ -2157,7 +2146,21 @@ static GtkWidget *make_tab_label(MultiTab *tab, GtkPositionType tab_pos)
             &tab->parent->best_tab_width);
     multi_tab_set_full_window_title(tab, tab->window_title_template,
             tab->window_title);
+#if GTK_CHECK_VERSION(3, 0, 0)
+    /* GtkBox will be replaced by GtkGrid one day, but at the moment it
+     * doesn't work properly because GtkNotebook has no homogeneous option
+     * and GtkGrid doesn't have pack_end to right-align close button.
+     */
+     /*
+    tab->label_box = gtk_grid_new();
+    gtk_orientable_set_orientation(GTK_ORIENTABLE(tab->label_box),
+            GTK_ORIENTATION_HORIZONTAL);
+    */
+    tab->label_box = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 4);
+    gtk_box_set_homogeneous(GTK_BOX(tab->label_box), FALSE);
+#else
     tab->label_box = gtk_hbox_new(FALSE, 4);
+#endif
     gtk_box_pack_start(GTK_BOX(tab->label_box), tab->label, TRUE, TRUE, 0);
     g_signal_connect(tab->label, "button-press-event",
             G_CALLBACK(tab_clicked_handler), tab);
@@ -2408,17 +2411,6 @@ MultiWinScrollBar_Position multi_win_set_scroll_bar_position(MultiWin * win,
     return win->scroll_bar_pos;
 }
 
-MultiWin *multi_win_get_win_under_pointer(void)
-{
-    int win_x, win_y;
-    GdkWindow *gdkwin = gdk_window_at_pointer(&win_x, &win_y);
-
-    if (!gdkwin)
-        return NULL;
-    return g_object_get_data(G_OBJECT(gdk_window_get_toplevel(gdkwin)),
-            "ROXTermWin");
-}
-
 GtkNotebook *multi_win_get_notebook(MultiWin *win)
 {
     return GTK_NOTEBOOK(win->notebook);
